#include "DBMWrapper.h"
#include "exception.h"

extern "C" {
#include <gdbm.h>
}

#include <sys/stat.h>
#include <string>
#include <boost/scoped_array.hpp>

class DBMImplementation
{
public:
  DBMImplementation( const std::string& filename )
  {
    boost::scoped_array<char> fn( new char[filename.size() + 1] );
    std::copy( filename.begin(), filename.end(), fn.get() );
    fn[ filename.size() ] = '\0';

    dbmFile_ = gdbm_open( fn.get(), 4096, GDBM_NEWDB, S_IWUSR | S_IRUSR | S_IRGRP | S_IROTH, 0 );
    if( !dbmFile_ )
      throw IOException( std::string("Unable to open DBM file '")
          + filename + "'." );
  }

  ~DBMImplementation()
  {
    gdbm_close( dbmFile_ );
  }

  void store( const std::string& key, const std::string& value )
  {
    DatumWrapper keyWrap( key );
    DatumWrapper valueWrap( value );
    int retVal = gdbm_store( dbmFile_, keyWrap(), valueWrap(), GDBM_INSERT );
    if( retVal != 0 )
      throw IOException( std::string( "Error storing key '" + key + 
        "', value '" + value + "' in the database." ) );
  }

private:
  GDBM_FILE dbmFile_;

  class DatumWrapper
  {
  public:
    DatumWrapper( const std::string& value )
      : array_( new char[value.size() + 1] )
    {
      std::copy( value.begin(), value.end(), array_.get() );
      array_[ value.size() ] = '\0';

      value_.dptr = array_.get();
      value_.dsize = value.size();
    }

    datum operator()() const
    {
      return value_;
    }

  private:
    datum value_;
    boost::scoped_array<char> array_;
  };
};

DBMWrapper::DBMWrapper( const std::string& filename )
  : pImpl_( new DBMImplementation(filename) )
{
}

DBMWrapper::~DBMWrapper()
{
}

void DBMWrapper::store( const std::string& key, const std::string& value )
{
  pImpl_->store( key, value );
}


