#ifndef __BIBTEXPARSER_H__
#define __BIBTEXPARSER_H__

// Wrappers for btparse, a library for parsing BiBTeX files.  
// Current version of btparse can be found here.
//    http://www.gerg.ca/software/btOOL/

#include <string>
#include <vector>



extern "C"
{
#include <btparse.h>
}


struct Author
{
public:
  Author( bt_name* name );

  Author( const std::string& first, const std::string& last )
    : first_(first), last_(last) {}

  std::string toString() const
  {
    std::string result( first_ );
    if( !von_.empty() )
    {
      result += " ";
      result += von_;
    }
    result += " ";
    result += last_;
    if( !jr_.empty() )
    {
      result += " ";
      result += jr_;
    }
    return result;
  }

  std::string first() const { return first_; }
  std::string von() const   { return von_; }
  std::string last() const  { return last_; }
  std::string jr() const    { return jr_; }

private:
  std::string first_;
  std::string von_;
  std::string last_;
  std::string jr_;
};

std::vector<Author> authors( const char* s );

class BTEntry
{
public:
  BTEntry( AST* entry );

  std::vector<Author> authors() const { return authors_; }
  std::string title() const { return title_; }
  unsigned int year() const { return year_; }
  unsigned int month() const { return month_; }
  std::string key() const { return key_; }
  std::string abstract() const { return abstract_; }
  std::string image() const { return image_; }
  std::vector<std::string> links() const { return links_; }
  std::string bibEntry() const;
	std::string sidebarImage() const { return sidebarImage_; }
	std::string sidebarText() const { return sidebarText_; }
  std::string journal() const { return this->journal_; }
  std::string booktitle() const { return this->bookTitle_; }
	std::string iconImage() const { return iconImage_; }

private:
  std::vector< Author > authors_;
  std::string title_;
  unsigned int year_;
  unsigned int month_;
  std::string key_;
  std::string type_;
  std::string abstract_;
  std::string image_;
  std::string journal_;
  std::string bookTitle_;

  std::string sidebarImage_;
  std::string sidebarText_;

	std::string iconImage_;

  std::vector<std::string> links_;

  typedef std::pair<std::string, std::string> StringPair;
  std::vector< StringPair > fields_;
};

std::vector<BTEntry> parseBiBTeX( const std::string& filename );

#endif

