#ifndef __EXCEPTION_H__
#define __EXCEPTION_H__

#include <boost/spirit/iterator.hpp>
#include <sstream>
#include <iostream>


template <typename T>
std::string toString( T value )
{
  std::ostringstream oss;
  oss << value;
  return oss.str();
}


class Exception
{
public:
  Exception( const std::string& msg )
    : msg_(msg)
  {
  }

  std::string message() const
  {
    return msg_;
  }

private:
   std::string msg_;
};

typedef boost::spirit::position_iterator< 
  boost::spirit::file_iterator<> > iterator_t;

class ParserException : public Exception
{
public:
  ParserException( const std::string& msg ) : Exception(msg) {}

  ParserException( const std::string& msg, const std::string::const_iterator& location )
    : Exception( msg ) {}

  ParserException( const std::string& msg, const iterator_t& location )
    : Exception( "Error in file '" + location.get_position().file + 
        "', line " + toString(location.get_position().line) + 
        ", column " + toString(location.get_position().column) + ": " + msg )
  {
  }
};

class IOException : public Exception
{
public:
  IOException( const std::string& msg ) : Exception(msg) {}
};

#endif


