/*****************************************************************************
Copyright (c) 2008, Christopher D. Twigg

All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
  list of conditions and the following disclaimer.  
* Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*******************************************************************************/

#include "homePages.h"
#include "exception.h"
#include <fstream>

// Reader for the homePages file.  homePages.txt has the following format:
// Firstname1 Lastname1 Homepage1
// Firstname2 Lastname2 Homepage2
// Firstname3 Lastname3 Homepage3
//
// e.g.:
// Christopher Twigg http://www.cs.cmu.edu/~cdtwigg
// Doug James http://www.cs.cmu.edu/~djames
// ...
//
// Hardcoding the location is clumsy, should change this
HomePages::HomePages( const std::string& homePagesLoc )
{
  // Read pages
  {
    std::ifstream ifs( homePagesLoc.c_str() );
    if( !ifs.good() )
      throw IOException( std::string("Unable to open homepages file '") + 
        homePagesLoc + std::string("'.") );

    while( ifs )
    {
      std::string first;
      std::string last;
      std::string page;
      ifs >> first >> std::ws >> last >> std::ws >> page >> std::ws;
      if( first.empty() || last.empty() || page.empty() )
         continue;
      HomePageMap::const_iterator itr = homePages_.insert( 
        std::make_pair( Author(first, last), page ) ).first;
      std::cout << "Found homepage for " << itr->first.toString() << ": " 
        << itr->second << std::endl;
    }
  }
}

std::string onlyAlpha( const std::string& name )
{
  std::string result;
  for( std::string::const_iterator itr = name.begin();
       itr != name.end();
       ++itr )
  {
    if( isalpha( *itr ) )
       result.push_back( *itr );
    else if( *itr == '\\' )
    {
       // Ignore escaped characters.
       ++itr;
       if( itr == name.end() )
         break;
    }
  }

  return result;
}

std::string HomePages::pageForName( const Author& name ) const
{
  Author simplified( onlyAlpha(name.first()), onlyAlpha(name.last()) );

  HomePageMap::const_iterator itr = homePages_.find( simplified );
  if( itr == homePages_.end() )
    return std::string();
  return itr->second;
}

