#ifndef __HTMLWRAPPERS_H__
#define __HTMLWRAPPERS_H__

#include <iostream>
#include <string>

// Global variable, this could easily get fixed though
extern std::string serverName;

inline std::string stripGraphics( const std::string& url )
{
  std::string::size_type pos = url.find( serverName );
  if( pos == std::string::npos )
    return url;
  else
    return std::string( url.begin() + serverName.size(), url.end() );
}


class HtmlThing
{
public:
  HtmlThing( std::ostream& out, std::string tag, bool endLine = false, std::string options = std::string() )
    : tag_(tag), endLine_(endLine), out_(out)
  {
    out_ << "<" << tag_;
    if( !options.empty() )
      out_ << " " << options;
    out_ << ">";
  }

  virtual ~HtmlThing()
  {
    out_ << "</" << tag_ << ">";
    if( endLine_ )
      out_ << std::endl;
  }

private:
  std::string tag_;
  bool endLine_;
  std::ostream& out_;
};

class TableRow : public HtmlThing
{
public:
  TableRow(std::ostream& out)
    : HtmlThing(out, "tr", true)
  {
  }

  virtual ~TableRow() {}
};

class TableCell : public HtmlThing
{
public:
  TableCell(std::ostream& out, bool endLine = false)
     : HtmlThing(out, "td", endLine) {}

  virtual ~TableCell() {}
};

class Link : public HtmlThing
{
public:
  Link( std::ostream& out, std::string address )
    : HtmlThing( out, "a", false, "href=\"" + stripGraphics(address) + "\"" )
  {
  }

  virtual ~Link() {}
};

#endif

