/*****************************************************************************
Copyright (c) 2008, Christopher D. Twigg

All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
  list of conditions and the following disclaimer.  
* Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*******************************************************************************/


#include "shortenedPublications.h"
#include "exception.h"

#include <boost/regex.hpp> 
#include <fstream>

ShortenedPublications::ShortenedPublications( const std::string& filename )
{
  std::ifstream ifs( filename.c_str() );
  if( !ifs )
    throw IOException( "Unable to open file: " + filename );

  while( ifs )
  {
    // this is kind of ugly, I admit
    while( ifs && ifs.get() != '\"' )
      ;
    if( !ifs )
       break;

    std::string regex;
    while( ifs )
    {
      int c = ifs.get();
      if( c == '\"' )
        break;
      if( isspace(c) )
      {
        regex.push_back( '\\' );
        regex.push_back( 's' );
        regex.push_back( '+' );
      }
      else
      {
        regex.push_back( c );
      }
    }
    if( !ifs )
       break;

    std::string shortName;
    while( ifs )
    {
      int c = ifs.get();
      if( c == '\n' || c == '\r' )
        break;
      shortName.push_back( c );
    }

    if( regex.empty() )
      continue;

    boost::regex compiledRegex(regex.c_str());
    shortenedNames.push_back( ShortenedName(compiledRegex, shortName) );

    std::cout << "Regular expression /" << compiledRegex << 
      "/ maps to \"" << shortName << "\"\n";
  }
}

std::string ShortenedPublications::shortName( const std::string& name ) const
{
//  std::cout << "Searching for '" << name << "'...\n";
  for( std::deque<ShortenedName>::const_iterator itr = shortenedNames.begin();
    itr != shortenedNames.end(); ++itr )
  {
//    std::cout << "  Comparing against /" << itr->first << "/... ";
    if( boost::regex_search( name, itr->first ) )
    {
//      std::cout << "match to " << itr->second << "\n";
      return itr->second;
    }
    else
    {
      ;
//      std::cout << "No match.\n";
    }
  }

  return std::string();
}

