/*****************************************************************************
Copyright (c) 2008, Christopher D. Twigg

All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
  list of conditions and the following disclaimer.  
* Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*******************************************************************************/


#include "utility.h"
#include "exception.h"

#include <boost/filesystem/operations.hpp>
#include <boost/filesystem/exception.hpp>

#include <fstream>

// Read a file and dump it to the output stream
void copyFile( const std::string& filename, std::ostream& os )
{
  std::ifstream ifs( filename.c_str() );

  if( !ifs.good() )
  {
    std::ostringstream oss;
    oss << "Unable to open file '" << filename << "' for reading." << std::endl;
    throw IOException( oss.str() );
  }

  std::ifstream::pos_type m = filelen( ifs );
  for( std::ifstream::pos_type i = ifs.tellg(); i < m; i += 1 )
  {
    int c = ifs.get();
    os.put( c );
  }
}

void swapFile( const std::string& filename, const boost::filesystem::path& outputPath )
{
  try
  {
    boost::filesystem::path curPath( outputPath / filename );
    boost::filesystem::path newPath( outputPath / (filename + ".new") );
    boost::filesystem::path oldPath( outputPath / (filename + ".old") );
    if( exists( oldPath ) )
      remove( oldPath );
    if( exists( curPath ) )
    {
      std::cout << "Moving " << curPath.native_file_string() <<
        " to " << oldPath.native_file_string() << "..." << std::flush;
      rename( curPath, oldPath );
      std::cout << "  Done." << std::endl;
    }

    std::cout << "Moving " << newPath.native_file_string() <<
      " to " << curPath.native_file_string() << "..." << std::flush;
    rename( newPath, curPath );
    std::cout << "  Done." << std::endl;
  }
  catch( boost::filesystem::filesystem_error& )
  {
    throw IOException( "Unexpected error moving '" + filename + ".new' into place." );
  }
}


